/*

Robot

Elever: <ditt namn>

Ursprunglig version: 2014-08-31/Staffan Melin

Kräver NewPing library
http://playground.arduino.cc/Code/NewPing

*/

//
// Vårt program behöver inkludera (lägga till) lite funktionalitet
// Det kallas för bibliotek.
//

#include <Servo.h>
#include <NewPing.h>
#include <SoftwareSerial.h>



//
// konstanter (värden som inte ändras i programmet)
//

int pinLed = 13; // utgång 13 är kopplat till en LED på Arduinon

const int pinVansterMotor1 = 2;
const int pinVansterMotor2 = 3;
const int pinVansterMotorFart = 6;

const int pinHogerMotor1 = 4;
const int pinHogerMotor2 = 5;
const int pinHogerMotorFart = 11;

const int pinServo = 10;

const int pinPingTrig = 19;
const int pinPingEcho = 18;
const int pingMax = 200; // hinder längre bort än pingMax centimeter kommer ej att ses

const int motorFart = 200; // hur snabbt ska motorerna gå?



//
// globala variabler (syns i hela programmet)
//

Servo myServo; // hjälper oss prata med servot
NewPing myPing(pinPingTrig, pinPingEcho, pingMax); // hjälper oss prata med avståndsmätaren

int avstandFramat; // uppnått avstånd framåt
int avstandVanster; // uppått avstånd åt vänster
int avstandHoger; // uppnått avstånd åt höger


//
// funktionen setup() körs EN GÅNG i början av programmet
//
void setup() {                

	pinMode(pinLed, OUTPUT);

	// avståndsmätare (ping = ultraljudssensor)
	// den LÄSER vi av (INPUT)
	// Raden NewPing myPing(...) ordnar så att portarna blir rätt för OUTPUT och INPUT
	// pinMode(pinPingTrig, OUTPUT);
	// pinMode(pinPingEcho, INPUT);

	// servo som styr avståndsmätaren
	// den STYR vi
	myServo.attach(pinServo);
	myServo.write(90); // ställ in servot på 90 grader (= rakt fram)

	// vänster motor
	// den STYR vi (OUTPUT)
	pinMode(pinVansterMotor1, OUTPUT);
	pinMode(pinVansterMotor2, OUTPUT);
	pinMode(pinVansterMotorFart, OUTPUT);

	// höger motor
	// den STYR vi (OUTPUT)
	pinMode(pinHogerMotor1, OUTPUT);
	pinMode(pinHogerMotor2, OUTPUT);
	pinMode(pinHogerMotorFart, OUTPUT);

	// sätt igång roboten framåt
	motorFramat();
}



//
// funktionen loop() körs om och om igen (tills strömmen bryts)
//
void loop() {

	digitalWrite(pinLed, 1);

	avstandFramat = pingMedelvarde();

	// vi bryr oss inte om avstånd som är 0 (för avståndsmätaren visar ibland fel)
	if (avstandFramat > 0)
	{
		// har vi ett hinder framför roboten?
		if (avstandFramat < 25)
		{
			motorStopp();

			// titta till vänster och mät avståndet
			myServo.write(10);
			delay(1000); // vänta 1 sekund så att servot hinner vrida sig
			avstandVanster = pingMedelvarde();

			// titta till höger och mät avståndet
			myServo.write(170);
			delay(1000);
			avstandHoger = pingMedelvarde();

			// vrid servot framåt
			myServo.write(90);
			delay(1000);

			// decide on a new direction
			
			// bestäm vad roboten ska göra nu

			// finns det hinder BÅDE till vänster och höger?
			if (avstandVanster < 20 && avstandHoger < 20) {
				motorBakat();
				delay(1000); // vänta ett tag medan roboten backar
				if (random(2) == 0)
					motorSvangVanster();
				else
					motorSvangHoger();
				delay(500); // vänta ett tag medan roboten svänger

			// är det friare till vänster?
			} else if (avstandVanster > avstandHoger)
			{
				motorSvangVanster();
				delay(1000); // vänta ett tag medan roboten svänger

			// tydligen är det friare till höger!
			} else {
				motorSvangHoger();
				delay(1000); // vänta ett tag medan roboten svänger
			}

			motorFramat();

		} else {
			// det finns inget hinder så kör framåt
			motorFramat();
		}

	}
}

void motorSvangHoger(void)
{
	// stoppa höger motor
	analogWrite(pinHogerMotorFart, 0);

	// kör vänster motor
	analogWrite(pinVansterMotorFart, motorFart);
	digitalWrite(pinVansterMotor1, LOW);
	digitalWrite(pinVansterMotor2, HIGH);
}

void motorSvangVanster(void)
{
	// stoppa vänster motor
	analogWrite(pinVansterMotorFart, 0);

	// kör höger motor
	analogWrite(pinHogerMotorFart, motorFart);
	digitalWrite(pinHogerMotor1, LOW);
	digitalWrite(pinHogerMotor2, HIGH);
}

void motorFramat(void)
{
	// kör vänster motor
	analogWrite(pinVansterMotorFart, motorFart);
	digitalWrite(pinVansterMotor1, LOW);
	digitalWrite(pinVansterMotor2, HIGH);

	// kör även höger motor
	analogWrite(pinHogerMotorFart, motorFart);
	digitalWrite(pinHogerMotor1, LOW);
	digitalWrite(pinHogerMotor2, HIGH);

}

void motorBakat(void)
{

	// kör vänster motor bakåt
	analogWrite(pinVansterMotorFart, motorFart);
	digitalWrite(pinVansterMotor1, HIGH);
	digitalWrite(pinVansterMotor2, LOW);

	// kör höger motor bakåt
	analogWrite(pinHogerMotorFart, motorFart);
	digitalWrite(pinHogerMotor1, HIGH);
	digitalWrite(pinHogerMotor2, LOW);

}

void motorStopp(void)
{
	// stoppa vänster motor
	analogWrite(pinVansterMotorFart, 0);
	digitalWrite(pinVansterMotor1, LOW);
	digitalWrite(pinVansterMotor2, LOW);

	// stoppa höger motor
	analogWrite(pinHogerMotorFart, 0);
	digitalWrite(pinHogerMotor1, LOW);
	digitalWrite(pinHogerMotor2, LOW);

}

// Den här funktionen mäter avståndet (i centimeter).
// Avståndsmätaren ger ibland fel värde, så mät 5 gånger och ta medelvärdet.

int pingMedelvarde()
{
	int i;
	int totAvstand = 0;

	for (i = 0; i < 5; i++)
	{
		totAvstand = totAvstand + myPing.ping_cm();
	}

	return (totAvstand / 5);
}
